$(document).ready(function() {

    $.ajax({
        url: url_redhatEmployeeListAjax,
        data: "",
        dataType: 'json',
        success: function(employeeList){
            empdata = employeeList;
            $("#timesheetSearch_txtEmployee").autocomplete(employeeList, {
                formatItem: function(item) {
                    return item.name;
                }, matchContains:"word"
            }).result(function(event, item) {
                $("#timesheetSearch_employeeNumber").val(item.id);
                $("#timesheetSearch_employeeNumber").trigger('change');
            });
            $("#timesheetSearch_txtEmployee").css("background-image", "none");
        }
    });
    
    $('#goToTimesheetStatusLink').click(function (event){
        event.preventDefault();
        $('#goToTimesheetStatusForm').submit();
    }); 

    $('#timesheetSearch_txtEmployee').keyup(function(){
        popluateAutocompleateOnType(empdata, $('#timesheetSearch_txtEmployee').val(), 'timesheetSearch_employeeNumber' );
    });
    
    $('#timesheetSearch_yearChoices').change(function(e){
    	getMonthListForSelectedYear($(this).val());
    });
    
    function getMonthListForSelectedYear(selectedYear) {
        
        $.ajax({
            url: url_getMonthListForTimesheetViewAjax,
            data: {
                year:selectedYear
            },
            dataType: 'json',
            success: function(data) {
                       
                $("#timesheetSearch_monthChoices option").remove();
                       
                $.each(data, function(k, v) {
                    $("#timesheetSearch_monthChoices").append("<option value=\"" + k  + "\">" + v  + "</option>");
                });
            }
        }); 
                
    }
    
    $("#editTimesheetForm").validate({
        onsubmit: true,
        onkeydown: false,
        onkeyup: false,
        onfocusin: false,
        focusInvalid: true,
        errorPlacement: function(error, element) {
            error.appendTo(element.next(".errorContainer"));
            error.appendTo(element.next().next(".errorContainer"));
            // error.appendTo("#validationMessage");
        }      
    });

    $.validator.addMethod("validateTime", function(value, element) {
        return validateTime(value, false);
    });

    $(".timesheetTextCell").each(function() {
        $(this).rules("add", {
            validateTime: true,
            messages: {
                validateTime: acceptedRange
            }
        });
    });
    
    $.validator.addMethod("maxlength", function (value, element, length) {
        value = value.replace(/(\r\n|\n|\r)/gm, '\r\n');
        return value == "" || value.length <= length;
    });
    
    $("textarea").each(function() {
        $(this).rules("add", {
            'maxlength': 255,
            messages: {
                'maxlength': commentMaxLengthValidationMessage
            }
        });
    });
});

function addDefaultRowValidations(row) {
    var field = row.find(".timesheetTextCell");
    
    field.each(function() {
        $(this).rules("add", {
            validateTime: true,
            messages: {
                validateTime: acceptedRange
            }
        });
    });
    
    row.find(".timesheetCommentCell").rules("add", {
        'maxlength': 255,
        messages: {
            'maxlength': commentMaxLengthValidationMessage
        }
    });
    
    addCustomRowValidations(row);
}

function updateTotalDuration(row) {
    var getTotal = new getRecordTotalSuper(row);
    var recordTotal = getTotal.getRecordTotal();
    var totalWorkCell = row.find('.timesheetTotalWorkCell');
    totalWorkCell.attr('value', recordTotal);
    totalWorkCell.html(recordTotal);
    totalWorkCell.next(':input[type=hidden]').val(recordTotal);
    totalWorkCell.trigger('change');
    updateTotalTimeSection();
};

function getRecordTotalSuper(row) {
    this.row = row;
}

function pad(number, length) {
    var str = '' + number;
    while (str.length < length) {
        str = '0' + str;
    }
    return str;
}

function getDurationDiff(totalDuration){

    var duration = totalDuration.split(":");

    if(duration.join('') <= 0){
        return false;
    } else {
        return true;
    }
}

function validateInAndOutTime(inTime, outTime){
    inTimeInitial = inTime;
    outTimeInitial = outTime;
    outTime = (outTime).split(":");
    inTime = (inTime).split(":");

    if(inTime == '' && outTime == '') {
        return true;
    }

    var fromdate = new Date();
    fromdate.setHours(inTime[0],inTime[1],0);

    var todate = new Date();
    todate.setHours( (outTime[0]) , (outTime[1]),0);

    var difference = todate - fromdate;
    var floatDeference	=	parseFloat(difference/3600000) ;

    if(floatDeference>0){
        return true;
    } else if (inTimeInitial == '00:00' && outTimeInitial == '00:00') {
        return true;
    } else {
        return false;
    }

}

function validateBreakInAndBreakOutTimeWithinInAndOutTime(inTime, outTime, breakInTime, breakOutTime) {
    var inTimeInitial = inTime;
    var outTimeInitial = outTime;

    var breakInTimeInitial = breakInTime;
    var breakOutTimeInitial = breakOutTime;

    outTime = (outTime).split(":");
    inTime = (inTime).split(":");

    breakOutTime = (breakOutTime).split(":");
    breakInTime = (breakInTime).split(":");

    if (inTime == '' && outTime == '' || breakInTime == '' && breakOutTime == '') {
        return true;
    }

    var inDate = new Date();
    inDate.setHours(inTime[0], inTime[1], 0);

    var outDate = new Date();
    outDate.setHours((outTime[0]), (outTime[1]), 0);

    var breakInDate = new Date();
    breakInDate.setHours(breakInTime[0], breakInTime[1], 0);

    var breakOutDate = new Date();
    breakOutDate.setHours((breakOutTime[0]), (breakOutTime[1]), 0);

    var inAndBreakOutDiff = breakOutDate - inDate;
    var breakInAndOutDiff = outDate - breakInDate;

    var inAndBreakOutFloatDeference = parseFloat(inAndBreakOutDiff / 3600000);
    var breakInAndOutFloatDeference = parseFloat(breakInAndOutDiff / 3600000);

    return inAndBreakOutFloatDeference >= 0 && breakInAndOutFloatDeference >= 0;

}

function correctTimeDurations(element){

    var timeString = element.val();
    var formatedTime = null;
                
    if (timeString == '') {
        return;
    }
        
    if (timeString == '24:00') {
        return '24:00';
    }
                                    
    if (timeString.indexOf(':') == -1) {
        element.val(createTimeFromString(timeString));
        return;
    }
                                    
    timeString = element.val();                    
    var timeSplitedArray = timeString.split(":");

    if(timeSplitedArray [1] == null ){
        timeSplitedArray [1] = 0;
    }
                
    if ((timeSplitedArray[0] == 0 && timeSplitedArray[1] == 0) || (timeSplitedArray[0] == 24 && timeSplitedArray[1] == 0)) {
        element.val('00:00');
        return;
    }
                                    
    strMin = '';
                                    
    if(timeSplitedArray [1].length < 2) {
        if(timeSplitedArray [1] <6) {
            strMin = timeSplitedArray [1] + '0'
        }else {
            strMin = '0'+ timeSplitedArray [1] 
        }
    }else {
        strMin = timeSplitedArray [1];
    }
                                    
    if(strMin == '') {
        strMin = '00';
    }
                                    
    formatedTime = pad(timeSplitedArray [0],2)+":"+ strMin;
                                  
    if(formatedTime != '00:00') {
        element.val(formatedTime);
    }else {
        element.val('');
    }
                                    
}

function validateTime(timeValue, required){
    var returnVal = true;
    time = (timeValue).split(":");
    if( timeValue.length == 5 ){

        if(timeValue.length >0){
            if( (time[1] == null) || (time[0] == null)){
                returnVal = false;
            }
        }

        if(time[0]>23){
            returnVal = false;
        }

        if(time[0] == 23 && time[1]>59){
            returnVal = false;
        } else {
            if(time[1]>59){
                returnVal = false;
            }
        }

        if(returnVal){
            durationError = false;
            return true;
        } else {
            durationError = true;
            return false;
        }

    } else if( (timeValue.length == 0) && required ){                           
        durationError = false;
        return true;
    } else if( timeValue.length == 0 ){
        durationError = false;
        return true;
    }
    else {
        durationError = true;
        return false;
    }

}

function createTimeFromString(timeString) {

    var formatedTime = null;

    if (isNaN(timeString)) {
        return timeString;
    }

    if (timeString == '0') {
        return '00:00';
    }

    if (timeString.match(/^0[01][6-9]$/)) {
        formatedTime = timeString.substr(0, 2) + ':0' + timeString.substr(2);
        return formatedTime;
    }

    if (timeString.match(/^0[2-9][6-9][0-9]$/)) {
        return timeString;
    }

    if (timeString.match(/^0[3-5]0$/)) {
        formatedTime = timeString.substr(0, 2) + ':' + timeString.substr(2) + '0';
        return formatedTime;
    }

    if (timeString.match(/^02[0-3]$/)) {
        formatedTime = timeString.replace(/^02/, '02:') + '0';
        return formatedTime;
    }

    if (timeString.match(/^01[6-9][0-9]$/)) {
        return timeString;
    }

    if (timeString.match(/^23[6-9]$/)) {
        formatedTime = timeString.replace(/^23/, '23:0');
        return formatedTime;
    }

    if (timeString.match(/^01[0-6]$/)) {
        formatedTime = timeString.substr(0, 2) + ':' + timeString.substr(2) + '0';
        return formatedTime;
    }

    if (timeString.match(/^0[0-2][0-5][0-9]$/)) {
        formatedTime = timeString.substr(0, 2) + ':' + timeString.substr(2);
        return formatedTime;
    }

    if (timeString.match(/^00[1-5]$/)) {
        formatedTime = timeString.replace(/^00/, '00:') + '0';
        return formatedTime;
    }

    if (timeString.match(/^0[2-5]0$/)) {
        formatedTime = timeString.replace(/^0/, '00:');
        return formatedTime;
    }

    if (timeString.match(/^00[6-9][0-9]/)) {
        return timeString;
    }

    timeString = timeString.replace(/^0/, '');

    var i = parseInt(timeString);
    var hourPart, minutePart;

    if (i < 1) {     
        formatedTime = '00:00';
        return formatedTime;
    } else if (i < 10) {
        formatedTime = '0' + i + ':00';
    } else if (i < 100) {
        if (i < 24) {
            formatedTime = ((i < 10) ? ('0' + i) : i) + ':00';
        } else {
            hourPart = Math.floor(i / 10);
            minutePart = i % 10;
            minutePart = (minutePart < 6) ? (minutePart + '0') : ('0' + minutePart);
            formatedTime = '0' + hourPart + ':' + minutePart;
        }
    } else if (i < 1000) {
        if (i < 236) {
            hourPart = Math.floor(i / 10);
            minutePart = i % 10;
            minutePart = (minutePart < 6) ? (minutePart + '0') : ('0' + minutePart);
            formatedTime = hourPart + ':' + minutePart;
        } else {
            hourPart = Math.floor(i / 100);
            minutePart = i % 100;
            if (minutePart < 60) {
                formatedTime = '0' + hourPart + ':' + ((minutePart < 9) ? '0' : '' ) + minutePart;
            } else {
                formatedTime = '';
            }
        }
    } else if (i < 10000) {
        if (i < 2360) {
            hourPart = Math.floor(i / 100);
            minutePart = i % 100;
            if (minutePart < 10) {
                formatedTime =  hourPart + ':0' + minutePart;
            } else if (minutePart < 60) {
                formatedTime =  hourPart + ':' + minutePart;
            } else {
                formatedTime = '';
            }
        } else {
            formatedTime = '';
        }
    } else {
        formatedTime = '';
    }

    if (formatedTime.match(/\:[0-9]$/)) {
        var timeParts = formatedTime.split(':');
        formatedTime = timeParts[0] + ':0' + timeParts[1]; 
    }

    return (formatedTime == '') ? timeString : formatedTime;
}

function updateTotalTimeSection() {
    var totalDuration = 0;
    $('input[name*="[totalWork]"]').each(function() {
        var value = $(this).val();
        if (value != '' && $(this).closest('tr').css('display') !== 'none') {
            var totalDurationParts = value.split(':');
            totalDuration += (totalDurationParts[0] * 60) + (totalDurationParts[1] * 1);
        }
    });

    var hours = Math.floor(totalDuration / 60);
    var minutes = totalDuration % 60;

    if(totalTimeRoundApplicable){
        $('.timesheet_table_total_section .totalTimeEnteredValue').html(pad(roundTOnearestHour(hours, minutes), 2) + ':00');
    }else{
        $('.timesheet_table_total_section .totalTimeEnteredValue').html(pad(hours, 2) + ':' + pad(minutes, 2));
    }
    
    updateTotalTimeWithLeave(totalDuration);
}

function updateTotalTimeWithLeave(totalDuration) {
    var value = $('input#totalLeave').val();
    var holidayValue = $('input#totalHoliday').val();
    
    var totalLeaveDurationPart = value.split(':');
    var totalLeaveDuration = (totalLeaveDurationPart[0] * 60) + (totalLeaveDurationPart[1] * 1);
    
    var totalHolidayDurationPart = holidayValue.split(':');
    var totalHolidayDuration = (totalHolidayDurationPart[0] * 60) + (totalHolidayDurationPart[1] * 1);
    
    var totalTime = totalDuration + totalLeaveDuration + totalHolidayDuration;

    var hours = Math.floor(totalTime / 60);
    var minutes = totalTime % 60;
    
     if(totalTimeRoundApplicable){
         $('#spanTotalTime').html(pad(roundTOnearestHour(hours, minutes), 2) + ':00' );
     }else{
         $('#spanTotalTime').html(pad(hours, 2) + ':' + pad(minutes, 2));
     }
    $('#totalWorkWithLeave').attr('value', (pad(hours, 2) + ':' + pad(minutes, 2)));
    
}

function roundTOnearestHour(hours, minutes){
    if(minutes >= 30){
        hours = hours + 1;
    }
    return  hours;
}